@@ -95,6 +95,8 @@ struct _CheeseCameraPrivate
 
   guint eos_timeout_id;
 
+  gchar *initial_name;
+
   CheeseCameraDeviceMonitor *monitor;
 };
 
@@ -1269,12 +1271,13 @@ cheese_camera_finalize (GObject *object)
   if (priv->photo_filename)
     g_free (priv->photo_filename);
   g_free (priv->current_effect_desc);
-  g_object_unref (priv->device);
+  g_clear_object (&priv->device);
   g_boxed_free (CHEESE_TYPE_VIDEO_FORMAT, priv->current_format);
 
   /* Free CheeseCameraDevice array */
   g_ptr_array_free (priv->camera_devices, TRUE);
 
+  g_free (priv->initial_name);
   g_clear_object (&priv->monitor);
 
   G_OBJECT_CLASS (cheese_camera_parent_class)->finalize (object);
@@ -1326,7 +1329,7 @@ cheese_camera_set_property (GObject *object, guint prop_id, const GValue *value,
       priv->video_texture = g_value_get_pointer (value);
       break;
     case PROP_DEVICE:
-      g_object_unref (priv->device);
+      g_clear_object (&priv->device);
       priv->device = g_value_dup_object (value);
       break;
     case PROP_FORMAT:
@@ -1474,7 +1477,7 @@ cheese_camera_init (CheeseCamera *camera)
 /**
  * cheese_camera_new:
  * @video_texture: a #ClutterTexture
- * @camera_device_node: (allow-none): the device node path
+ * @name: (allow-none): the name of the device
  * @x_resolution: the resolution width
  * @y_resolution: the resolution height
  *
@@ -1483,25 +1486,23 @@ cheese_camera_init (CheeseCamera *camera)
  * Returns: a new #CheeseCamera
  */
 CheeseCamera *
-cheese_camera_new (ClutterActor *video_texture, const gchar *camera_device_node,
+cheese_camera_new (ClutterActor *video_texture, const gchar *name,
                    gint x_resolution, gint y_resolution)
 {
-  CheeseCamera      *camera;
-  CheeseVideoFormat format = { x_resolution, y_resolution };
+    CheeseCamera      *camera;
+    CheeseVideoFormat format = { x_resolution, y_resolution };
 
-  if (camera_device_node)
-  {
-    camera = g_object_new (CHEESE_TYPE_CAMERA, "video-texture", video_texture,
-                           "device-node", camera_device_node,
-                           "format", &format, NULL);
-  }
-  else
-  {
     camera = g_object_new (CHEESE_TYPE_CAMERA, "video-texture", video_texture,
                            "format", &format, NULL);
-  }
 
-  return camera;
+    if (name)
+    {
+        CheeseCameraPrivate *priv = cheese_camera_get_instance_private (camera);
+
+        priv->initial_name = g_strdup (name);
+    }
+
+    return camera;
 }
 
 /**
@@ -1559,6 +1560,22 @@ cheese_camera_setup (CheeseCamera *camera, CheeseCameraDevice *device, GError **
   {
     cheese_camera_set_device (camera, device);
   }
+  else
+  {
+    guint i;
+
+    for (i = 0; i < priv->num_camera_devices; i++)
+    {
+      device = g_ptr_array_index (priv->camera_devices, i);
+
+      if (g_strcmp0 (cheese_camera_device_get_name (device),
+                     priv->initial_name) == 0)
+      {
+        cheese_camera_set_device (camera, device);
+        break;
+      }
+    }
+  }
 
 
   if ((priv->camerabin = gst_element_factory_make ("camerabin", "camerabin")) == NULL)
